package com.linbit.linstor.api.rest.v1.serializer;

import com.linbit.ImplementationError;
import com.linbit.InvalidNameException;
import com.linbit.ValueOutOfRangeException;
import com.linbit.linstor.LinstorParsingUtils;
import com.linbit.linstor.api.ApiCallRc;
import com.linbit.linstor.api.interfaces.AutoSelectFilterApi;
import com.linbit.linstor.api.interfaces.RscDfnLayerDataApi;
import com.linbit.linstor.api.interfaces.RscLayerDataApi;
import com.linbit.linstor.api.interfaces.VlmDfnLayerDataApi;
import com.linbit.linstor.api.interfaces.VlmLayerDataApi;
import com.linbit.linstor.api.pojo.BCacheRscPojo;
import com.linbit.linstor.api.pojo.CacheRscPojo;
import com.linbit.linstor.api.pojo.DrbdRscPojo;
import com.linbit.linstor.api.pojo.ExosConnectionMapPojo;
import com.linbit.linstor.api.pojo.ExosDefaultsPojo;
import com.linbit.linstor.api.pojo.ExosEnclosureEventPojo;
import com.linbit.linstor.api.pojo.ExosEnclosureHealthPojo;
import com.linbit.linstor.api.pojo.ExternalFilePojo;
import com.linbit.linstor.api.pojo.LinstorRemotePojo;
import com.linbit.linstor.api.pojo.LuksRscPojo;
import com.linbit.linstor.api.pojo.MaxVlmSizeCandidatePojo;
import com.linbit.linstor.api.pojo.NetInterfacePojo;
import com.linbit.linstor.api.pojo.NvmeRscPojo;
import com.linbit.linstor.api.pojo.OpenflexRscPojo;
import com.linbit.linstor.api.pojo.RscPojo;
import com.linbit.linstor.api.pojo.S3RemotePojo;
import com.linbit.linstor.api.pojo.StorageRscPojo;
import com.linbit.linstor.api.pojo.VlmDfnPojo;
import com.linbit.linstor.api.pojo.WritecacheRscPojo;
import com.linbit.linstor.api.pojo.backups.BackupInfoPojo;
import com.linbit.linstor.api.pojo.backups.BackupInfoStorPoolPojo;
import com.linbit.linstor.api.pojo.backups.BackupInfoVlmPojo;
import com.linbit.linstor.api.rest.v1.serializer.JsonGenTypes.AutoSelectFilter;
import com.linbit.linstor.api.rest.v1.serializer.JsonGenTypes.ExosDefaults;
import com.linbit.linstor.core.apis.BackupApi;
import com.linbit.linstor.core.apis.BackupApi.BackupS3Api;
import com.linbit.linstor.core.apis.BackupApi.BackupVlmApi;
import com.linbit.linstor.core.apis.BackupApi.BackupVlmS3Api;
import com.linbit.linstor.core.apis.NetInterfaceApi;
import com.linbit.linstor.core.apis.NodeApi;
import com.linbit.linstor.core.apis.ResourceApi;
import com.linbit.linstor.core.apis.ResourceConnectionApi;
import com.linbit.linstor.core.apis.ResourceDefinitionApi;
import com.linbit.linstor.core.apis.ResourceGroupApi;
import com.linbit.linstor.core.apis.SnapshotApi;
import com.linbit.linstor.core.apis.SnapshotDefinitionListItemApi;
import com.linbit.linstor.core.apis.SnapshotShippingListItemApi;
import com.linbit.linstor.core.apis.SnapshotVolumeDefinitionApi;
import com.linbit.linstor.core.apis.StorPoolApi;
import com.linbit.linstor.core.apis.StorPoolDefinitionApi;
import com.linbit.linstor.core.apis.VolumeApi;
import com.linbit.linstor.core.apis.VolumeDefinitionApi;
import com.linbit.linstor.core.apis.VolumeGroupApi;
import com.linbit.linstor.core.identifier.NodeName;
import com.linbit.linstor.core.identifier.ResourceName;
import com.linbit.linstor.core.identifier.SharedStorPoolName;
import com.linbit.linstor.core.identifier.VolumeNumber;
import com.linbit.linstor.core.objects.Node;
import com.linbit.linstor.core.objects.Resource;
import com.linbit.linstor.core.objects.ResourceConnection;
import com.linbit.linstor.core.objects.ResourceDefinition;
import com.linbit.linstor.core.objects.Snapshot;
import com.linbit.linstor.core.objects.SnapshotDefinition;
import com.linbit.linstor.core.objects.Volume;
import com.linbit.linstor.core.objects.VolumeDefinition;
import com.linbit.linstor.core.objects.VolumeGroup;
import com.linbit.linstor.satellitestate.SatelliteResourceState;
import com.linbit.linstor.satellitestate.SatelliteState;
import com.linbit.linstor.satellitestate.SatelliteVolumeState;
import com.linbit.linstor.stateflags.FlagsHelper;
import com.linbit.linstor.storage.interfaces.layers.drbd.DrbdRscObject;
import com.linbit.linstor.storage.kinds.DeviceLayerKind;
import com.linbit.linstor.storage.kinds.DeviceProviderKind;
import com.linbit.linstor.storage.kinds.ExtTools;
import com.linbit.linstor.storage.kinds.ExtToolsInfo;
import com.linbit.utils.Base64;
import com.linbit.utils.Pair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.stream.Collectors;

import com.google.common.net.UrlEscapers;

public class Json
{
    public static String deviceProviderKindAsString(DeviceProviderKind deviceProviderKind)
    {
        return deviceProviderKind.name();
    }

    private static String getLayerTypeString(DeviceLayerKind deviceLayerKind)
    {
        return deviceLayerKind.name();
    }

    public static JsonGenTypes.Node apiToNode(
        NodeApi nodeApi
    )
    {
        JsonGenTypes.Node nd = new JsonGenTypes.Node();
        nd.name = nodeApi.getName();
        nd.type = nodeApi.getType();
        nd.connection_status = nodeApi.connectionStatus().toString();
        nd.props = nodeApi.getProps();
        nd.flags = Node.Flags.toStringList(nodeApi.getFlags());

        List<JsonGenTypes.NetInterface> netIfsList =
            nodeApi.getNetInterfaces().stream().map(Json::apiToNetInterface).collect(Collectors.toList());
        NetInterfaceApi activeStltConn = nodeApi.getActiveStltConn();
        if (activeStltConn != null)
        {
            for (JsonGenTypes.NetInterface netInterface : netIfsList)
            {
                netInterface.is_active = netInterface.uuid.equalsIgnoreCase(activeStltConn.getUuid().toString()) ?
                    Boolean.TRUE :
                    Boolean.FALSE;
            }
        }

        nd.net_interfaces = netIfsList;
        nd.uuid = nodeApi.getUuid().toString();
        nd.resource_layers = nodeApi.getDeviceLayerKindNames();
        nd.storage_providers = nodeApi.getDeviceProviderKindNames();
        nd.unsupported_layers = nodeApi.getUnsupportedLayersWithReasons();
        nd.unsupported_providers = nodeApi.getUnsupportedProvidersWithReasons();
        nd.eviction_timestamp = nodeApi.getEvictionTimestamp();
        return nd;
    }

    public static JsonGenTypes.NetInterface apiToNetInterface(NetInterfaceApi netIfApi)
    {
        JsonGenTypes.NetInterface netif = new JsonGenTypes.NetInterface();
        netif.name = netIfApi.getName();
        netif.address = netIfApi.getAddress();
        if (netIfApi.isUsableAsSatelliteConnection())
        {
            netif.satellite_encryption_type = netIfApi.getSatelliteConnectionEncryptionType();
            netif.satellite_port = netIfApi.getSatelliteConnectionPort();
        }
        netif.is_active = Boolean.FALSE; // default value
        netif.uuid = netIfApi.getUuid().toString();
        return netif;
    }

    public static NetInterfaceApi netInterfacetoApi(JsonGenTypes.NetInterface netif)
    {
        return new NetInterfacePojo(
            null,
            netif.name,
            netif.address,
            netif.satellite_port,
            netif.satellite_encryption_type
        );
    }

    public static JsonGenTypes.StoragePoolDefinition storPoolDfnApiToStoragePoolDefinition(
        StorPoolDefinitionApi apiData
    )
    {
        JsonGenTypes.StoragePoolDefinition storPoolDfn = new JsonGenTypes.StoragePoolDefinition();
        storPoolDfn.storage_pool_name = apiData.getName();
        storPoolDfn.props = apiData.getProps();
        return storPoolDfn;
    }

    public static List<JsonGenTypes.ApiCallRc> apiCallRcToJson(ApiCallRc apiCallRc)
    {
        return apiCallRc.getEntries().stream()
            .map(apiCallRcEntry ->
            {
               JsonGenTypes.ApiCallRc jsonApiCallRc = new JsonGenTypes.ApiCallRc();
              jsonApiCallRc.message = apiCallRcEntry.getMessage();
              jsonApiCallRc.cause = apiCallRcEntry.getCause();
              jsonApiCallRc.correction = apiCallRcEntry.getCorrection();
              jsonApiCallRc.details = apiCallRcEntry.getDetails();
              jsonApiCallRc.error_report_ids = new ArrayList<>(apiCallRcEntry.getErrorIds());
              jsonApiCallRc.obj_refs = apiCallRcEntry.getObjRefs();
              jsonApiCallRc.ret_code = apiCallRcEntry.getReturnCode();
               return jsonApiCallRc;
            }).collect(Collectors.toList());
    }

    public static JsonGenTypes.StoragePool storPoolApiToStoragePool(
        StorPoolApi storPoolApi
    )
    {
        JsonGenTypes.StoragePool storPoolData = new JsonGenTypes.StoragePool();
        storPoolData.storage_pool_name = storPoolApi.getStorPoolName();
        storPoolData.node_name = storPoolApi.getNodeName();
        storPoolData.provider_kind = Json.deviceProviderKindAsString(
            storPoolApi.getDeviceProviderKind()
        );
        storPoolData.props = storPoolApi.getStorPoolProps();
        storPoolData.static_traits = storPoolApi.getStorPoolStaticTraits();
        storPoolData.free_capacity = storPoolApi.getFreeCapacity().orElse(null);
        storPoolData.total_capacity = storPoolApi.getTotalCapacity().orElse(null);
        storPoolData.free_space_mgr_name = storPoolApi.getFreeSpaceManagerName();
        storPoolData.uuid = storPoolApi.getStorPoolUuid().toString();
        storPoolData.reports = apiCallRcToJson(storPoolApi.getReports());
        storPoolData.supports_snapshots = storPoolApi.supportsSnapshots();
        storPoolData.external_locking = storPoolApi.isExternalLocking();

        return storPoolData;
    }

    public static JsonGenTypes.DrbdResourceDefinitionLayer pojoToDrbdRscDfnLayer(
        DrbdRscPojo.DrbdRscDfnPojo drbdRscDfnPojo
    )
    {
        JsonGenTypes.DrbdResourceDefinitionLayer drbdResourceDefinitionLayer = new
            JsonGenTypes.DrbdResourceDefinitionLayer();
        drbdResourceDefinitionLayer.resource_name_suffix = drbdRscDfnPojo.getRscNameSuffix();
        drbdResourceDefinitionLayer.peer_slots = (int) drbdRscDfnPojo.getPeerSlots();
        drbdResourceDefinitionLayer.al_stripes = drbdRscDfnPojo.getAlStripes();
        drbdResourceDefinitionLayer.al_stripe_size_kib = drbdRscDfnPojo.getAlStripeSize();
        drbdResourceDefinitionLayer.port = drbdRscDfnPojo.getPort();
        drbdResourceDefinitionLayer.transport_type = drbdRscDfnPojo.getTransportType();
        drbdResourceDefinitionLayer.secret = drbdRscDfnPojo.getSecret();
        drbdResourceDefinitionLayer.down = drbdRscDfnPojo.isDown();
        return drbdResourceDefinitionLayer;
    }

    public static JsonGenTypes.OpenflexResourceDefinitionLayer pojoToOpenflexRscDfnLayer(
        OpenflexRscPojo.OpenflexRscDfnPojo ofRscDfnPojo
        )
    {
        JsonGenTypes.OpenflexResourceDefinitionLayer ofResourceDefinitionLayer = new
            JsonGenTypes.OpenflexResourceDefinitionLayer();
        ofResourceDefinitionLayer.resource_name_suffix = ofRscDfnPojo.getRscNameSuffix();
        ofResourceDefinitionLayer.nqn = ofRscDfnPojo.getNqn();
        return ofResourceDefinitionLayer;
    }

    public static JsonGenTypes.ResourceDefinition apiToResourceDefinition(
        ResourceDefinitionApi rscDfnApi,
        boolean withVlmDfns
    )
    {
        JsonGenTypes.ResourceDefinition rscDfn = new JsonGenTypes.ResourceDefinition();
        rscDfn.name = rscDfnApi.getResourceName();
        if (rscDfnApi.getExternalName() != null)
        {
            rscDfn.external_name = new String(rscDfnApi.getExternalName(), StandardCharsets.UTF_8);
        }
        rscDfn.flags = ResourceDefinition.Flags.toStringList(rscDfnApi.getFlags());
        rscDfn.props = rscDfnApi.getProps();
        rscDfn.uuid = rscDfnApi.getUuid().toString();

        rscDfn.layer_data = new ArrayList<>();

        if (withVlmDfns)
        {
            rscDfn.volume_definitions = new ArrayList<>();
            for (VolumeDefinitionApi vlmDfnApi : rscDfnApi.getVlmDfnList())
            {
                rscDfn.volume_definitions.add(apiToVolumeDefinition(vlmDfnApi));
            }
        }

        for (Pair<String, RscDfnLayerDataApi> layer : rscDfnApi.getLayerData())
        {
            JsonGenTypes.ResourceDefinitionLayer rscDfnLayerData = new JsonGenTypes.ResourceDefinitionLayer();
            rscDfnLayerData.type = layer.objA;

            if (layer.objB != null)
            {
                switch (layer.objB.getLayerKind())
                {
                    case DRBD:
                        DrbdRscPojo.DrbdRscDfnPojo drbdRscDfnPojo = (DrbdRscPojo.DrbdRscDfnPojo) layer.objB;
                        rscDfnLayerData.data = pojoToDrbdRscDfnLayer(drbdRscDfnPojo);
                        break;
                    case OPENFLEX:
                        OpenflexRscPojo.OpenflexRscDfnPojo ofRscDfnPojo = (OpenflexRscPojo.OpenflexRscDfnPojo) layer.objB;
                        rscDfnLayerData.data = pojoToOpenflexRscDfnLayer(ofRscDfnPojo);
                        break;
                    case LUKS:
                    case STORAGE:
                    case NVME:
                    case WRITECACHE:
                    case CACHE:
                    case BCACHE:
                    default:
                        throw new ImplementationError("Not implemented kind case");
                }
            }
            rscDfn.layer_data.add(rscDfnLayerData);
        }
        rscDfn.resource_group_name = rscDfnApi.getResourceGroup().getName();
        return rscDfn;
    }

    public static JsonGenTypes.DrbdVolumeDefinition pojoToDrbdVolumeDefinition(
        DrbdRscPojo.DrbdVlmDfnPojo drbdVlmDfnPojo
    )
    {
        JsonGenTypes.DrbdVolumeDefinition drbdVlmDfn = new JsonGenTypes.DrbdVolumeDefinition();
        drbdVlmDfn.resource_name_suffix = drbdVlmDfnPojo.getRscNameSuffix();
        drbdVlmDfn.volume_number = drbdVlmDfnPojo.getVlmNr();
        drbdVlmDfn.minor_number = drbdVlmDfnPojo.getMinorNr();
        return drbdVlmDfn;
    }

    public static JsonGenTypes.VolumeDefinition apiToVolumeDefinition(
        VolumeDefinitionApi vlmDfnApi
    )
    {
        JsonGenTypes.VolumeDefinition vlmDfn = new JsonGenTypes.VolumeDefinition();
        vlmDfn.volume_number = vlmDfnApi.getVolumeNr();
        vlmDfn.size_kib = vlmDfnApi.getSize();
        vlmDfn.props = vlmDfnApi.getProps();
        vlmDfn.flags = FlagsHelper.toStringList(
            VolumeDefinition.Flags.class,
            vlmDfnApi.getFlags()
        );
        vlmDfn.uuid = vlmDfnApi.getUuid().toString();

        vlmDfn.layer_data = new ArrayList<>();
        for (Pair<String, VlmDfnLayerDataApi> layer : vlmDfnApi.getVlmDfnLayerData())
        {
            JsonGenTypes.VolumeDefinitionLayer volumeDefinitionLayerData = new JsonGenTypes.VolumeDefinitionLayer();
            volumeDefinitionLayerData.type = layer.objA;

            if (layer.objB != null)
            {
                switch (layer.objB.getLayerKind())
                {
                    case DRBD:
                        DrbdRscPojo.DrbdVlmDfnPojo drbdVlmDfnPojo = (DrbdRscPojo.DrbdVlmDfnPojo) layer.objB;
                        volumeDefinitionLayerData.data = pojoToDrbdVolumeDefinition(drbdVlmDfnPojo);
                        break;
                    case LUKS:
                    case STORAGE:
                    case NVME:
                    case OPENFLEX:
                    case WRITECACHE:
                    case CACHE:
                    case BCACHE:
                    default:
                        throw new ImplementationError("Not implemented Kind case");
                }
            }
            vlmDfn.layer_data.add(volumeDefinitionLayerData);
        }
        return vlmDfn;
    }

    public static VolumeDefinitionApi volumeDefinitionToApi(
        JsonGenTypes.VolumeDefinition vlmDfn
    )
    {
        ArrayList<Pair<String, VlmDfnLayerDataApi>> layerData = new ArrayList<>(); // empty will be ignored
        return new VlmDfnPojo(
            null,
            vlmDfn.volume_number,
            vlmDfn.size_kib,
            FlagsHelper.fromStringList(VolumeDefinition.Flags.class, vlmDfn.flags),
            vlmDfn.props,
            layerData
        );
    }

    public static JsonGenTypes.DrbdResource pojoToDrbdResource(DrbdRscPojo drbdRscPojo)
    {
        JsonGenTypes.DrbdResource drbdResource = new JsonGenTypes.DrbdResource();
        drbdResource.drbd_resource_definition = pojoToDrbdRscDfnLayer(drbdRscPojo.getDrbdRscDfn());
        drbdResource.node_id = drbdRscPojo.getNodeId();
        drbdResource.peer_slots = (int) drbdRscPojo.getPeerSlots();
        drbdResource.al_stripes = drbdRscPojo.getAlStripes();
        drbdResource.al_size = drbdRscPojo.getAlStripeSize();
        drbdResource.flags = FlagsHelper.toStringList(DrbdRscObject.DrbdRscFlags.class, drbdRscPojo.getFlags());
        drbdResource.drbd_volumes = drbdRscPojo.getVolumeList().stream()
            .map(Json::pojoToDrbdVolume)
            .collect(Collectors.toList());
        drbdResource.promotion_score = drbdRscPojo.getPromotionScore();
        drbdResource.may_promote = drbdRscPojo.mayPromote();
        return drbdResource;
    }

    public static JsonGenTypes.StorageResource pojoToStorageResource(StorageRscPojo storageRscPojo)
    {
        JsonGenTypes.StorageResource storageResource = new JsonGenTypes.StorageResource();
        storageResource.storage_volumes = storageRscPojo.getVolumeList().stream()
            .map(Json::apiToStorageVolume)
            .collect(Collectors.toList());
        return storageResource;
    }

    public static JsonGenTypes.LUKSResource pojoToLUKSResource(LuksRscPojo luksRscPojo)
    {
        JsonGenTypes.LUKSResource luksResource = new JsonGenTypes.LUKSResource();
        luksResource.luks_volumes = luksRscPojo.getVolumeList().stream()
            .map(Json::pojoToLUKSVolume)
            .collect(Collectors.toList());
        return luksResource;
    }

    public static JsonGenTypes.NVMEResource pojoToNVMEResource(NvmeRscPojo nvmeRscPojo)
    {
        JsonGenTypes.NVMEResource nvmeResource = new JsonGenTypes.NVMEResource();
        nvmeResource.nvme_volumes = nvmeRscPojo.getVolumeList().stream()
            .map(Json::pojoToNVMEVolume)
            .collect(Collectors.toList());
        return nvmeResource;
    }

    public static JsonGenTypes.OpenflexResource pojoToOpenflexResource(OpenflexRscPojo ofRscPojo)
    {
        JsonGenTypes.OpenflexResource ofResource = new JsonGenTypes.OpenflexResource();
        ofResource.openflex_volumes = ofRscPojo.getVolumeList().stream()
            .map(Json::pojoToOpenflexVolume)
            .collect(Collectors.toList());
        return ofResource;
    }

    public static JsonGenTypes.WritecacheResource pojoToWritecacheResource(WritecacheRscPojo writecacheRscPojo)
    {
        JsonGenTypes.WritecacheResource writecacheResource = new JsonGenTypes.WritecacheResource();
        writecacheResource.writecache_volumes = writecacheRscPojo.getVolumeList().stream()
            .map(Json::pojoToWritecacheVolume)
            .collect(Collectors.toList());
        return writecacheResource;
    }

    public static JsonGenTypes.CacheResource pojoToCacheResource(CacheRscPojo cacheRscPojo)
    {
        JsonGenTypes.CacheResource cacheResource = new JsonGenTypes.CacheResource();
        cacheResource.cache_volumes = cacheRscPojo.getVolumeList().stream()
            .map(Json::pojoToCacheVolume)
            .collect(Collectors.toList());
        return cacheResource;
    }

    public static JsonGenTypes.BCacheResource pojoToBCacheResource(BCacheRscPojo bcacheRscPojo)
    {
        JsonGenTypes.BCacheResource bcacheResource = new JsonGenTypes.BCacheResource();
        bcacheResource.bcache_volumes = bcacheRscPojo.getVolumeList().stream()
            .map(Json::pojoToBCacheVolume)
            .collect(Collectors.toList());
        return bcacheResource;
    }

    public static JsonGenTypes.ResourceLayer apiToResourceLayer(RscLayerDataApi rscLayerDataApi)
    {
        JsonGenTypes.ResourceLayer resourceLayer = new JsonGenTypes.ResourceLayer();
        resourceLayer.resource_name_suffix = rscLayerDataApi.getRscNameSuffix();
        resourceLayer.children = rscLayerDataApi.getChildren().stream()
            .map(Json::apiToResourceLayer)
            .collect(Collectors.toList());
        resourceLayer.type = getLayerTypeString(rscLayerDataApi.getLayerKind());
        switch (rscLayerDataApi.getLayerKind())
        {
            case DRBD:
                DrbdRscPojo drbdRscPojo = (DrbdRscPojo) rscLayerDataApi;
                resourceLayer.drbd = pojoToDrbdResource(drbdRscPojo);
                break;
            case STORAGE:
                StorageRscPojo storageRscPojo = (StorageRscPojo) rscLayerDataApi;
                resourceLayer.storage = pojoToStorageResource(storageRscPojo);
                break;
            case LUKS:
                LuksRscPojo luksRscPojo = (LuksRscPojo) rscLayerDataApi;
                resourceLayer.luks = pojoToLUKSResource(luksRscPojo);
                break;
            case NVME:
                NvmeRscPojo nvmeRscPojo = (NvmeRscPojo) rscLayerDataApi;
                resourceLayer.nvme = pojoToNVMEResource(nvmeRscPojo);
                break;
            case OPENFLEX:
                OpenflexRscPojo ofRscPojo = (OpenflexRscPojo) rscLayerDataApi;
                resourceLayer.openflex = pojoToOpenflexResource(ofRscPojo);
                break;
            case WRITECACHE:
                WritecacheRscPojo writecacheRscPojo = (WritecacheRscPojo) rscLayerDataApi;
                resourceLayer.writecache = pojoToWritecacheResource(writecacheRscPojo);
                break;
            case CACHE:
                CacheRscPojo cacheRscPojo = (CacheRscPojo) rscLayerDataApi;
                resourceLayer.cache = pojoToCacheResource(cacheRscPojo);
                break;
            case BCACHE:
                BCacheRscPojo bcacheRscPojo = (BCacheRscPojo) rscLayerDataApi;
                resourceLayer.bcache = pojoToBCacheResource(bcacheRscPojo);
            default:
        }
        return resourceLayer;
    }

    public static JsonGenTypes.ResourceWithVolumes apiToResourceWithVolumes(
        ResourceApi rscApi,
        Map<NodeName, SatelliteState> satelliteStates,
        boolean withVolumes
    )
    {
        JsonGenTypes.ResourceWithVolumes rsc = new JsonGenTypes.ResourceWithVolumes();
        rsc.name = rscApi.getName();
        rsc.node_name = rscApi.getNodeName();
        rsc.flags = FlagsHelper.toStringList(Resource.Flags.class, rscApi.getFlags());
        rsc.props = rscApi.getProps();
        rsc.layer_object = apiToResourceLayer(rscApi.getLayerData());
        rsc.uuid = rscApi.getUuid().toString();
        rscApi.getCreateTimestamp().ifPresent(d -> rsc.create_timestamp = d.getTime());

        rsc.shared_name = getSharedName(rscApi);

        if (withVolumes)
        {
            rsc.volumes = rscApi.getVlmList().stream().map(vlmApi ->
                {
                    JsonGenTypes.Volume vlmData = Json.apiToVolume(vlmApi);
                    JsonGenTypes.VolumeState vlmState = null;
                    try
                    {
                        final ResourceName rscNameRes = new ResourceName(rsc.name);
                        final NodeName linNodeName = new NodeName(rsc.node_name);
                        if (satelliteStates.containsKey(linNodeName) &&
                            satelliteStates.get(linNodeName)
                                .getResourceStates().containsKey(rscNameRes))
                        {
                            SatelliteResourceState satResState = satelliteStates
                                .get(linNodeName)
                                .getResourceStates()
                                .get(rscNameRes);

                            VolumeNumber vlmNumber = new VolumeNumber(vlmData.volume_number);
                            if (satResState.getVolumeStates().containsKey(vlmNumber))
                            {
                                vlmState = new JsonGenTypes.VolumeState();
                                SatelliteVolumeState satVlmState = satResState.getVolumeStates().get(vlmNumber);
                                vlmState.disk_state = satVlmState.getDiskState();
                            }
                        }
                    }
                    catch (InvalidNameException | ValueOutOfRangeException ignored)
                    {
                    }
                    vlmData.state = vlmState;
                    return vlmData;
                }).collect(Collectors.toList());
        }

        try
        {
            final ResourceName rscNameRes = new ResourceName(rscApi.getName());
            final NodeName linNodeName = new NodeName(rscApi.getNodeName());
            if (satelliteStates.containsKey(linNodeName) &&
                satelliteStates.get(linNodeName)
                    .getResourceStates().containsKey(rscNameRes))
            {
                SatelliteResourceState satResState = satelliteStates
                    .get(linNodeName).getResourceStates().get(rscNameRes);
                rsc.state = new JsonGenTypes.ResourceState();
                rsc.state.in_use = satResState.isInUse();

                if (rscApi.getLayerData().getLayerKind() == DeviceLayerKind.DRBD)
                {
                    rsc.layer_object.drbd.connections = new HashMap<>();
                    for (Map.Entry<NodeName, String> entry : satResState.getConnectionStates()
                        .getOrDefault(linNodeName, new HashMap<>()).entrySet())
                    {
                        JsonGenTypes.DrbdConnection con = new JsonGenTypes.DrbdConnection();
                        con.connected = entry.getValue().equals("Connected");
                        con.message = entry.getValue();
                        rsc.layer_object.drbd.connections.put(entry.getKey().displayValue, con);
                    }
                }
            }
        }
        catch (InvalidNameException ignored)
        {
        }
        return rsc;
    }

    private static String getSharedName(ResourceApi rscApiRef)
    {
        LinkedList<RscLayerDataApi> toExplore = new LinkedList<>();
        toExplore.add(rscApiRef.getLayerData());
        while (!toExplore.isEmpty())
        {
            RscLayerDataApi cur = toExplore.removeFirst();
            if (cur.getLayerKind().equals(DeviceLayerKind.STORAGE))
            {
                if (cur.getVolumeList().isEmpty())
                {
                    return null; // no volumes, no sharedName
                }
                String freeSpaceManagerName = cur.getVolumeList().get(0).getStorPoolApi().getFreeSpaceManagerName();
                if (SharedStorPoolName.isShared(freeSpaceManagerName))
                {
                    return freeSpaceManagerName;
                }
                else
                {
                    return null; // do not return if sharedSpName is not shared
                }
            }
            toExplore.addAll(cur.getChildren());
        }
        return null;
    }

    public static JsonGenTypes.Resource apiToResource(
        ResourceApi rscApi,
        Map<NodeName, SatelliteState> satelliteStates
    )
    {
        return apiToResourceWithVolumes(rscApi, satelliteStates, false);
    }

    public static ResourceApi resourceToApi(JsonGenTypes.Resource resource)
    {
        return new RscPojo(
            resource.name,
            resource.node_name,
            FlagsHelper.fromStringList(Resource.Flags.class, resource.flags),
            resource.props
        );
    }

    public static JsonGenTypes.DrbdVolume pojoToDrbdVolume(
        DrbdRscPojo.DrbdVlmPojo drbdVlmPojo
    )
    {
        JsonGenTypes.DrbdVolume drbdVolume = new JsonGenTypes.DrbdVolume();
        drbdVolume.drbd_volume_definition = pojoToDrbdVolumeDefinition(drbdVlmPojo.getDrbdVlmDfn());
        drbdVolume.device_path = drbdVlmPojo.getDevicePath();
        drbdVolume.backing_device = drbdVlmPojo.getBackingDisk();
        drbdVolume.meta_disk = drbdVlmPojo.getMetaDisk();
        drbdVolume.allocated_size_kib = drbdVlmPojo.getAllocatedSize();
        drbdVolume.usable_size_kib = drbdVlmPojo.getUsableSize();
        drbdVolume.disk_state = drbdVlmPojo.getDiskState();
        drbdVolume.ext_meta_stor_pool = drbdVlmPojo.getExternalMetaDataStorPool();
        return drbdVolume;
    }

    public static JsonGenTypes.LUKSVolume pojoToLUKSVolume(
        LuksRscPojo.LuksVlmPojo luksVlmPojo
    )
    {
        JsonGenTypes.LUKSVolume luksVolume = new JsonGenTypes.LUKSVolume();
        luksVolume.volume_number = luksVlmPojo.getVlmNr();
        luksVolume.device_path = luksVlmPojo.getDevicePath();
        luksVolume.backing_device = luksVlmPojo.getBackingDevice();
        luksVolume.allocated_size_kib = luksVlmPojo.getAllocatedSize();
        luksVolume.usable_size_kib = luksVlmPojo.getUsableSize();
        luksVolume.opened = luksVlmPojo.isOpen();
        luksVolume.disk_state = luksVlmPojo.getDiskState();
        return luksVolume;
    }

    public static JsonGenTypes.NVMEVolume pojoToNVMEVolume(
        NvmeRscPojo.NvmeVlmPojo nvmeVlmPojo
    )
    {
        JsonGenTypes.NVMEVolume nvmeVolume = new JsonGenTypes.NVMEVolume();
        nvmeVolume.volume_number = nvmeVlmPojo.getVlmNr();
        nvmeVolume.device_path = nvmeVlmPojo.getDevicePath();
        nvmeVolume.backing_device = nvmeVlmPojo.getBackingDisk();
        nvmeVolume.allocated_size_kib = nvmeVlmPojo.getAllocatedSize();
        nvmeVolume.usable_size_kib = nvmeVlmPojo.getUsableSize();
        nvmeVolume.disk_state = nvmeVlmPojo.getDiskState();
        return nvmeVolume;
    }

    public static JsonGenTypes.OpenflexVolume pojoToOpenflexVolume(
        OpenflexRscPojo.OpenflexVlmPojo ofVlmPojo
    )
    {
        JsonGenTypes.OpenflexVolume nvmeVolume = new JsonGenTypes.OpenflexVolume();
        nvmeVolume.volume_number = ofVlmPojo.getVlmNr();
        nvmeVolume.device_path = ofVlmPojo.getDevicePath();
        nvmeVolume.allocated_size_kib = ofVlmPojo.getAllocatedSize();
        nvmeVolume.usable_size_kib = ofVlmPojo.getUsableSize();
        nvmeVolume.disk_state = ofVlmPojo.getDiskState();
        return nvmeVolume;
    }

    public static JsonGenTypes.WritecacheVolume pojoToWritecacheVolume(
        WritecacheRscPojo.WritecacheVlmPojo writecacheVlmPojo
    )
    {
        JsonGenTypes.WritecacheVolume writecacheVolume = new JsonGenTypes.WritecacheVolume();
        writecacheVolume.volume_number = writecacheVlmPojo.getVlmNr();
        writecacheVolume.device_path = writecacheVlmPojo.getDevicePath();
        writecacheVolume.device_path_cache = writecacheVlmPojo.getDevicePathCache();
        writecacheVolume.allocated_size_kib = writecacheVlmPojo.getAllocatedSize();
        writecacheVolume.usable_size_kib = writecacheVlmPojo.getUsableSize();
        writecacheVolume.disk_state = writecacheVlmPojo.getDiskState();
        return writecacheVolume;
    }

    public static JsonGenTypes.CacheVolume pojoToCacheVolume(
        CacheRscPojo.CacheVlmPojo cacheVlmPojo
    )
    {
        JsonGenTypes.CacheVolume cacheVolume = new JsonGenTypes.CacheVolume();
        cacheVolume.volume_number = cacheVlmPojo.getVlmNr();
        cacheVolume.device_path = cacheVlmPojo.getDevicePath();
        cacheVolume.device_path_cache = cacheVlmPojo.getDevicePathCache();
        cacheVolume.allocated_size_kib = cacheVlmPojo.getAllocatedSize();
        cacheVolume.usable_size_kib = cacheVlmPojo.getUsableSize();
        cacheVolume.disk_state = cacheVlmPojo.getDiskState();
        return cacheVolume;
    }

    public static JsonGenTypes.BCacheVolume pojoToBCacheVolume(
        BCacheRscPojo.BCacheVlmPojo bcacheVlmPojo
    )
    {
        JsonGenTypes.BCacheVolume cacheVolume = new JsonGenTypes.BCacheVolume();
        cacheVolume.volume_number = bcacheVlmPojo.getVlmNr();
        cacheVolume.device_path = bcacheVlmPojo.getDevicePath();
        cacheVolume.device_path_cache = bcacheVlmPojo.getDevicePathCache();
        cacheVolume.allocated_size_kib = bcacheVlmPojo.getAllocatedSize();
        cacheVolume.usable_size_kib = bcacheVlmPojo.getUsableSize();
        cacheVolume.disk_state = bcacheVlmPojo.getDiskState();
        return cacheVolume;
    }

    public static JsonGenTypes.StorageVolume apiToStorageVolume(
        VlmLayerDataApi vlmLayerDataApi
    )
    {
        JsonGenTypes.StorageVolume storageVolume = new JsonGenTypes.StorageVolume();
        storageVolume.volume_number = vlmLayerDataApi.getVlmNr();
        storageVolume.device_path = vlmLayerDataApi.getDevicePath();
        storageVolume.allocated_size_kib = vlmLayerDataApi.getAllocatedSize();
        storageVolume.usable_size_kib = vlmLayerDataApi.getUsableSize();
        storageVolume.disk_state = vlmLayerDataApi.getDiskState();
        return storageVolume;
    }

    public static JsonGenTypes.Volume apiToVolume(VolumeApi vlmApi)
    {
        JsonGenTypes.Volume volume = new JsonGenTypes.Volume();
        volume.volume_number = vlmApi.getVlmNr();
        volume.storage_pool_name = vlmApi.getStorPoolName();
        volume.provider_kind = deviceProviderKindAsString(vlmApi.getStorPoolDeviceProviderKind());

        volume.device_path = vlmApi.getDevicePath();
        volume.allocated_size_kib = vlmApi.getAllocatedSize().orElse(null);

        volume.props = vlmApi.getVlmProps();
        volume.flags = FlagsHelper.toStringList(Volume.Flags.class, vlmApi.getFlags());
        volume.uuid = vlmApi.getVlmUuid().toString();
        volume.reports = apiCallRcToJson(vlmApi.getReports());

        volume.layer_data_list = new ArrayList<>();

        for (Pair<String, VlmLayerDataApi> layerData : vlmApi.getVlmLayerData())
        {
            JsonGenTypes.VolumeLayer volumeLayerData = new JsonGenTypes.VolumeLayer();
            volumeLayerData.type = getLayerTypeString(layerData.objB.getLayerKind());

            switch (layerData.objB.getLayerKind())
            {
                case DRBD:
                    DrbdRscPojo.DrbdVlmPojo drbdVlm = (DrbdRscPojo.DrbdVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToDrbdVolume(drbdVlm);
                    break;
                case STORAGE:
                    volumeLayerData.data = apiToStorageVolume(layerData.objB);
                    break;
                case LUKS:
                    LuksRscPojo.LuksVlmPojo luksVlmPojo = (LuksRscPojo.LuksVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToLUKSVolume(luksVlmPojo);
                    break;
                case NVME:
                    NvmeRscPojo.NvmeVlmPojo nvmeVlmPojo = (NvmeRscPojo.NvmeVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToNVMEVolume(nvmeVlmPojo);
                    break;
                case OPENFLEX:
                    OpenflexRscPojo.OpenflexVlmPojo ofVlmPojo = (OpenflexRscPojo.OpenflexVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToOpenflexVolume(ofVlmPojo);
                    break;
                case WRITECACHE:
                    WritecacheRscPojo.WritecacheVlmPojo writecacheVlmPojo = (WritecacheRscPojo.WritecacheVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToWritecacheVolume(writecacheVlmPojo);
                    break;
                case CACHE:
                    CacheRscPojo.CacheVlmPojo cacheVlmPojo = (CacheRscPojo.CacheVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToCacheVolume(cacheVlmPojo);
                    break;
                case BCACHE:
                    BCacheRscPojo.BCacheVlmPojo bcacheVlmPojo = (BCacheRscPojo.BCacheVlmPojo) layerData.objB;
                    volumeLayerData.data = pojoToBCacheVolume(bcacheVlmPojo);
                    break;
                default:
            }

            volume.layer_data_list.add(volumeLayerData);
        }
        return volume;
    }

    public static class AutoSelectFilterData implements AutoSelectFilterApi
    {
        private final JsonGenTypes.AutoSelectFilter autoSelectFilter;

        public AutoSelectFilterData(JsonGenTypes.AutoSelectFilter autoSelectFilterRef)
        {
            autoSelectFilter = autoSelectFilterRef;
            if (
                (autoSelectFilterRef.storage_pool_list == null || autoSelectFilterRef.storage_pool_list.isEmpty()) &&
                    autoSelectFilterRef.storage_pool != null
            )
            {
                autoSelectFilterRef.storage_pool_list = Collections.singletonList(autoSelectFilterRef.storage_pool);
            }
        }

        @Override
        public Integer getReplicaCount()
        {
            return autoSelectFilter.place_count;
        }

        @Override
        public Integer getAdditionalReplicaCount()
        {
            return autoSelectFilter.additional_place_count;
        }

        @Override
        public List<String> getNodeNameList()
        {
            return autoSelectFilter.node_name_list;
        }

        @Override
        public List<String> getStorPoolNameList()
        {
            return autoSelectFilter.storage_pool_list;
        }

        @Override
        public List<String> getStorPoolDisklessNameList()
        {
            return autoSelectFilter.storage_pool_diskless_list;
        }

        @Override
        public List<String> getDoNotPlaceWithRscList()
        {
            return autoSelectFilter.not_place_with_rsc;
        }

        @Override
        public String getDoNotPlaceWithRscRegex()
        {
            return autoSelectFilter.not_place_with_rsc_regex;
        }

        @Override
        public List<String> getReplicasOnSameList()
        {
            return autoSelectFilter.replicas_on_same;
        }

        @Override
        public List<String> getReplicasOnDifferentList()
        {
            return autoSelectFilter.replicas_on_different;
        }

        @Override
        public List<DeviceLayerKind> getLayerStackList()
        {
            return autoSelectFilter.layer_stack != null ?
                LinstorParsingUtils.asDeviceLayerKind(autoSelectFilter.layer_stack) : null;
        }

        @Override
        public List<DeviceProviderKind> getProviderList()
        {
            return autoSelectFilter.provider_list != null ?
                LinstorParsingUtils.asProviderKind(autoSelectFilter.provider_list) : null;
        }

        @Override
        public Boolean getDisklessOnRemaining()
        {
            return autoSelectFilter.diskless_on_remaining;
        }

        @Override
        public List<String> skipAlreadyPlacedOnNodeNamesCheck()
        {
            return null;
        }

        @Override
        public Boolean skipAlreadyPlacedOnAllNodeCheck()
        {
            return null;
        }

        @Override
        public String getDisklessType()
        {
            return autoSelectFilter.diskless_type;
        }

        @Override
        public Map<ExtTools, ExtToolsInfo.Version> getRequiredExtTools()
        {
            return null;
        }
    }

    public static JsonGenTypes.SnapshotVolumeDefinition apiToSnapshotVolumeDefinition(
        SnapshotVolumeDefinitionApi snapshotVlmDfnApi
    )
    {
        JsonGenTypes.SnapshotVolumeDefinition snapshotVolumeDefinition =
            new JsonGenTypes.SnapshotVolumeDefinition();

        snapshotVolumeDefinition.volume_number = snapshotVlmDfnApi.getVolumeNr();
        snapshotVolumeDefinition.size_kib = snapshotVlmDfnApi.getSize();

        return snapshotVolumeDefinition;
    }

    public static JsonGenTypes.SnapshotNode apiToSnapshotNode(SnapshotApi snapshotApi)
    {
        JsonGenTypes.SnapshotNode nodeSnap = new JsonGenTypes.SnapshotNode();
        nodeSnap.node_name = snapshotApi.getNodeName();
        nodeSnap.snapshot_name = snapshotApi.getSnaphotDfn().getSnapshotName();
        nodeSnap.flags = FlagsHelper.toStringList(Snapshot.Flags.class, snapshotApi.getFlags());
        nodeSnap.uuid = snapshotApi.getSnapshotUuid().toString();
        snapshotApi.getCreateTimestamp().ifPresent(d -> nodeSnap.create_timestamp = d.getTime());
        return nodeSnap;
    }

    public static JsonGenTypes.Snapshot apiToSnapshot(
        SnapshotDefinitionListItemApi snapshotDfnListItemApi
    )
    {
        JsonGenTypes.Snapshot snapshot = new JsonGenTypes.Snapshot();
        snapshot.name = snapshotDfnListItemApi.getSnapshotName();
        snapshot.resource_name = snapshotDfnListItemApi.getRscDfn().getResourceName();
        snapshot.nodes = snapshotDfnListItemApi.getNodeNames();
        snapshot.props = snapshotDfnListItemApi.getProps();
        snapshot.flags = FlagsHelper.toStringList(SnapshotDefinition.Flags.class, snapshotDfnListItemApi.getFlags());
        snapshot.volume_definitions = snapshotDfnListItemApi.getSnapshotVlmDfnList().stream()
            .map(Json::apiToSnapshotVolumeDefinition)
            .collect(Collectors.toList());
        snapshot.uuid = snapshotDfnListItemApi.getUuid().toString();
        snapshot.snapshots = snapshotDfnListItemApi.getSnapshots().stream()
            .map(Json::apiToSnapshotNode)
            .collect(Collectors.toList());
        return snapshot;
    }

    public static JsonGenTypes.SnapshotShippingStatus apiToSnapshotShipping(
        SnapshotShippingListItemApi snapshotShippingListItemApi
    )
    {
        JsonGenTypes.SnapshotShippingStatus shippingStatus = new JsonGenTypes.SnapshotShippingStatus();
        shippingStatus.snapshot = apiToSnapshot(snapshotShippingListItemApi);
        shippingStatus.from_node_name = snapshotShippingListItemApi.getSourceNodeName();
        shippingStatus.to_node_name = snapshotShippingListItemApi.getTargetNodeName();
        shippingStatus.status = snapshotShippingListItemApi.getShippingStatus();
        return shippingStatus;
    }

    public static JsonGenTypes.ResourceConnection apiToResourceConnection(
        ResourceConnectionApi rscConnApi
    )
    {
        JsonGenTypes.ResourceConnection resourceConnection = new JsonGenTypes.ResourceConnection();
        resourceConnection.node_a = rscConnApi.getSourceNodeName();
        resourceConnection.node_b = rscConnApi.getTargetNodeName();
        resourceConnection.props = rscConnApi.getProps();
        resourceConnection.flags = FlagsHelper.toStringList(
            ResourceConnection.Flags.class, rscConnApi.getFlags()
        );
        resourceConnection.port = rscConnApi.getPort();
        return resourceConnection;
    }

    public static JsonGenTypes.Candidate pojoToCandidate(MaxVlmSizeCandidatePojo pojo)
    {
        JsonGenTypes.Candidate candidate = new JsonGenTypes.Candidate();
        candidate.storage_pool = pojo.getStorPoolDfnApi().getName();
        candidate.max_volume_size_kib = pojo.getMaxVlmSize();
        candidate.node_names = pojo.getNodeNames();
        candidate.all_thin = pojo.areAllThin();
        return candidate;
    }

    public static JsonGenTypes.MaxVolumeSizes pojoToMaxVolumeSizes(List<MaxVlmSizeCandidatePojo> candidatePojos)
    {
        JsonGenTypes.MaxVolumeSizes maxVolumeSizes = new JsonGenTypes.MaxVolumeSizes();
        maxVolumeSizes.candidates = candidatePojos.stream().map(Json::pojoToCandidate).collect(Collectors.toList());
        return maxVolumeSizes;
    }

    public static JsonGenTypes.KeyValueStore apiToKeyValueStore(com.linbit.linstor.core.apis.KvsApi kvsApi)
    {
        JsonGenTypes.KeyValueStore keyValueStore = new JsonGenTypes.KeyValueStore();
        keyValueStore.name = kvsApi.getName();
        keyValueStore.props = kvsApi.getProps();
        return keyValueStore;
    }

    public static JsonGenTypes.ResourceGroup apiToResourceGroup(
        ResourceGroupApi rscGrpApi
    )
    {
        JsonGenTypes.ResourceGroup rscGrp = new JsonGenTypes.ResourceGroup();
        rscGrp.name = rscGrpApi.getName();
        if (rscGrpApi.getDescription() != null)
        {
            rscGrp.description = rscGrpApi.getDescription();
        }
        rscGrp.props = rscGrpApi.getProps();
        rscGrp.uuid = rscGrpApi.getUuid().toString();

        AutoSelectFilterApi autoSelectApi = rscGrpApi.getAutoSelectFilter();
        if (autoSelectApi != null)
        {
            AutoSelectFilter auto_select_filter = new AutoSelectFilter();
            if (autoSelectApi.getReplicaCount() != null)
            {
                auto_select_filter.place_count = autoSelectApi.getReplicaCount();
            }
            if (autoSelectApi.getStorPoolNameList() != null && autoSelectApi.getStorPoolNameList().size() == 1)
            {
                auto_select_filter.storage_pool = autoSelectApi.getStorPoolNameList().get(0);
            }
            auto_select_filter.storage_pool_list = autoSelectApi.getStorPoolNameList();
            auto_select_filter.not_place_with_rsc = autoSelectApi.getDoNotPlaceWithRscList();
            auto_select_filter.not_place_with_rsc_regex = autoSelectApi.getDoNotPlaceWithRscRegex();
            auto_select_filter.replicas_on_same = autoSelectApi.getReplicasOnSameList();
            auto_select_filter.replicas_on_different = autoSelectApi.getReplicasOnDifferentList();
            auto_select_filter.layer_stack = autoSelectApi.getLayerStackList().stream()
                .map(Json::getLayerTypeString).collect(Collectors.toList());
            auto_select_filter.provider_list = autoSelectApi.getProviderList().stream()
                .map(Json::deviceProviderKindAsString).collect(Collectors.toList());
            auto_select_filter.diskless_on_remaining = autoSelectApi.getDisklessOnRemaining();

            rscGrp.select_filter = auto_select_filter;
        }
        return rscGrp;
    }

    public static JsonGenTypes.VolumeGroup apiToVolumeGroup(
        VolumeGroupApi vlmGrpApi
    )
    {
        JsonGenTypes.VolumeGroup vlmGrp = new JsonGenTypes.VolumeGroup();
        vlmGrp.volume_number = vlmGrpApi.getVolumeNr();
        vlmGrp.props = vlmGrpApi.getProps();
        vlmGrp.uuid = vlmGrpApi.getUUID().toString();
        vlmGrp.flags = FlagsHelper.toStringList(VolumeGroup.Flags.class, vlmGrpApi.getFlags());
        return vlmGrp;
    }

    public static ExosDefaults apiToExosDefaults(ExosDefaultsPojo pojo)
    {
        JsonGenTypes.ExosDefaults json = new JsonGenTypes.ExosDefaults();
        json.password = pojo.getPassword();
        json.password_env = pojo.getPasswordEnv();
        json.username = pojo.getUsername();
        json.username_env = pojo.getUsernameEnv();
        return json;
    }

    public static JsonGenTypes.ExosEnclosureHealth apiToExosEnclosure(
        ExosEnclosureHealthPojo pojo
    )
    {
        JsonGenTypes.ExosEnclosureHealth json = new JsonGenTypes.ExosEnclosureHealth();
        json.name = pojo.getEnclosureName();
        json.ctrl_a_ip = pojo.getCtrlAIp();
        json.ctrl_b_ip = pojo.getCtrlBIp();
        json.health = pojo.getHealth();
        json.health_reason = pojo.getHealthReason();

        return json;
    }

    public static JsonGenTypes.ExosEnclosureEvent apiToExosEnclosureEvent(
        ExosEnclosureEventPojo pojo
    )
    {
        JsonGenTypes.ExosEnclosureEvent json = new JsonGenTypes.ExosEnclosureEvent();
        json.additional_information = pojo.getAdditionalInformation();
        json.controller = pojo.getController();
        json.event_id = pojo.getExosEventId();
        json.message = pojo.getMessage();
        json.recommended_action = pojo.getRecommendedAction();
        json.severity = pojo.getSeverity();
        json.time_stamp = pojo.getTimeStampStr();
        json.time_stamp_numeric = pojo.getTimeStampNumeric();

        return json;
    }

    public static JsonGenTypes.ExosConnectionMap apiToExosConnectionMap(
        ExosConnectionMapPojo pojo
    )
    {
        JsonGenTypes.ExosConnectionMap json = new JsonGenTypes.ExosConnectionMap();
        json.node_name = pojo.getNodeName();
        json.enclosure_name = pojo.getEnclosureName();
        json.connections = pojo.getConnections();
        return json;
    }

    public static JsonGenTypes.ExternalFile apiToExternalFile(
        ExternalFilePojo pojo,
        boolean includeContent
    )
    {
        JsonGenTypes.ExternalFile json = new JsonGenTypes.ExternalFile();
        json.path = pojo.getFileName();
        if (includeContent)
        {
            json.content = Base64.encode(pojo.getContent());
        }
        return json;
    }

    public static JsonGenTypes.ResourceDefinitionCloneStarted resourceDefCloneStarted(
        String srcRscName,
        String clonedName,
        ApiCallRc messages
    )
    {
        JsonGenTypes.ResourceDefinitionCloneStarted response = new JsonGenTypes.ResourceDefinitionCloneStarted();
        response.location = String.format(
            "/v1/resource-definitions/%s/clone/%s",
            srcRscName,
            UrlEscapers.urlPathSegmentEscaper().escape(clonedName));
        response.source_name = srcRscName;
        response.clone_name = clonedName;
        response.messages = Json.apiCallRcToJson(messages);
        return response;
    }

    public static Map<String, JsonGenTypes.Backup> apiToBackup(Map<String, BackupApi> backups)
    {
        Map<String, JsonGenTypes.Backup> jsonBackups = new TreeMap<>();
        for (BackupApi backup : backups.values())
        {
            JsonGenTypes.Backup jsonBackup = new JsonGenTypes.Backup();

            jsonBackup.id = backup.getId();
            jsonBackup.start_time = backup.getStartTime();
            jsonBackup.start_timestamp = backup.getStartTimestamp();
            jsonBackup.finished_time = backup.getFinishedTime();
            jsonBackup.finished_timestamp = backup.getFinishedTimestamp();
            jsonBackup.origin_rsc = backup.getResourceName();
            jsonBackup.origin_node = backup.getOriginNodeName();
            jsonBackup.success = backup.successful();
            jsonBackup.shipping = backup.isShipping();
            jsonBackup.restorable = backup.isRestoreable();
            // jsonBackup.fail_messages = backup.getfail_messages();

            jsonBackup.vlms = new ArrayList<>();
            for (BackupVlmApi backupVlmApi : backup.getVlms().values())
            {
                JsonGenTypes.BackupVolumes backupVlm = new JsonGenTypes.BackupVolumes();
                backupVlm.vlm_nr = backupVlmApi.getVlmNr();
                backupVlm.finished_time = backupVlmApi.getFinishedTime();
                backupVlm.finished_timestamp = backupVlmApi.getFinishedTimestamp();
                BackupVlmS3Api s3VlmApi = backupVlmApi.getS3();
                if (s3VlmApi != null)
                {
                    backupVlm.s3 = new JsonGenTypes.BackupVolumesS3();
                    backupVlm.s3.key = s3VlmApi.getS3Key();
                }
                jsonBackup.vlms.add(backupVlm);
            }

            BackupS3Api s3Api = backup.getS3();
            if (s3Api != null)
            {
                jsonBackup.s3 = new JsonGenTypes.BackupS3();
                jsonBackup.s3.meta_name = s3Api.getMetaName();
            }

            String basedOn = backup.getBasedOnId();
            if (basedOn != null)
            {
                jsonBackup.based_on_id = basedOn;
            }
            jsonBackups.put(backup.getId(), jsonBackup);
        }
        return jsonBackups;
    }

    public static JsonGenTypes.BackupInfo apiToBackupInfo(BackupInfoPojo pojo)
    {
        JsonGenTypes.BackupInfo json = new JsonGenTypes.BackupInfo();
        json.rsc = pojo.getRscName();
        json.full = pojo.getFullBackupName();
        json.latest = pojo.getLatestBackupName();
        json.count = pojo.getBackupCount();
        json.dl_size_kib = pojo.getDlSizeKib();
        json.alloc_size_kib = pojo.getAllocSizeKib();
        json.storpools = new ArrayList<>();
        for (BackupInfoStorPoolPojo storPoolPojo : pojo.getStorpools())
        {
            JsonGenTypes.BackupInfoStorPool storPoolJson = new JsonGenTypes.BackupInfoStorPool();
            storPoolJson.name = storPoolPojo.getStorPoolName();
            storPoolJson.provider_kind = storPoolPojo.getProviderKind().name();
            storPoolJson.target_name = storPoolPojo.getTargetStorPoolName();
            storPoolJson.remaining_space_kib = storPoolPojo.getRemainingSpaceKib();
            storPoolJson.vlms = new ArrayList<>();
            for (BackupInfoVlmPojo vlmPojo : storPoolPojo.getVlms())
            {
                JsonGenTypes.BackupInfoVolume vlmJson = new JsonGenTypes.BackupInfoVolume();
                vlmJson.name = vlmPojo.getBackupVlmName();
                vlmJson.layer_type = vlmPojo.getLayerType().name();
                vlmJson.dl_size_kib = vlmPojo.getDlSizeKib();
                vlmJson.alloc_size_kib = vlmPojo.getAllocSizeKib();
                vlmJson.usable_size_kib = vlmPojo.getUseSizeKib();
                storPoolJson.vlms.add(vlmJson);
            }
            json.storpools.add(storPoolJson);
        }
        return json;
    }

    public static JsonGenTypes.S3Remote apiToS3Remote(S3RemotePojo pojo)
    {
        JsonGenTypes.S3Remote json = new JsonGenTypes.S3Remote();
        json.remote_name = pojo.getRemoteName();
        json.endpoint = pojo.getEndpoint();
        json.bucket = pojo.getBucket();
        json.region = pojo.getRegion();

        return json;
    }

    public static JsonGenTypes.LinstorRemote apiToLinstorRemote(LinstorRemotePojo pojo)
    {
        JsonGenTypes.LinstorRemote json = new JsonGenTypes.LinstorRemote();
        json.remote_name = pojo.getRemoteName();
        json.url = pojo.getUrl();
        // encrypted passphrase intentionally not serialized

        return json;
    }

    private Json()
    {
    }
}
