CREATE TABLE RESOURCE_GROUPS
(
    UUID CHARACTER(36) NOT NULL,
    RESOURCE_GROUP_NAME VARCHAR(48) NOT NULL, -- PK
    RESOURCE_GROUP_DSP_NAME VARCHAR(48) NOT NULL,
    DESCRIPTION VARCHAR(1024) NULL, -- nullable
    LAYER_STACK VARCHAR(1024) NULL, -- nullable, json list of strings
    REPLICA_COUNT INT NULL, -- nullable, if null, this rscGrp is not usable with auto-place
    POOL_NAME VARCHAR(48) NULL, -- nullable, no foreign key, to allow referencing not yet or no longer existing storage pools
    DO_NOT_PLACE_WITH_RSC_REGEX VARCHAR(4096) NULL, -- nullable
    DO_NOT_PLACE_WITH_RSC_LIST VARCHAR(4096) NULL, -- nullable, json list of strings
    REPLICAS_ON_SAME BLOB NULL, -- nullable, json list of strings
    REPLICAS_ON_DIFFERENT BLOB NULL, -- nullable, json list of strings
    ALLOWED_PROVIDER_LIST VARCHAR(1024) NULL, -- nullable, json list of strings
    DISKLESS_ON_REMAINING BOOLEAN NULL, -- nullable
    CONSTRAINT PK_RG PRIMARY KEY (RESOURCE_GROUP_NAME),
    CONSTRAINT UNQ_RG_UUID UNIQUE (UUID),
    CONSTRAINT CHK_RG_NAME CHECK (UPPER(RESOURCE_GROUP_NAME) = RESOURCE_GROUP_NAME AND LENGTH(RESOURCE_GROUP_NAME) >= 2),
    CONSTRAINT CHK_RG_DSP_NAME CHECK (UPPER(RESOURCE_GROUP_DSP_NAME) = RESOURCE_GROUP_NAME)
);

CREATE TABLE VOLUME_GROUPS
(
    UUID CHARACTER(36) NOT NULL,
    RESOURCE_GROUP_NAME VARCHAR(48) NOT NULL, -- part of PK; FK to RESOURCE_GROUPS
    VLM_NR INTEGER NOT NULL, -- part of PK
    CONSTRAINT PK_VG PRIMARY KEY (RESOURCE_GROUP_NAME, VLM_NR),
    CONSTRAINT FK_VG_RT FOREIGN KEY (RESOURCE_GROUP_NAME) REFERENCES RESOURCE_GROUPS(RESOURCE_GROUP_NAME) ON DELETE CASCADE,
    CONSTRAINT UNQ_VG_UUID UNIQUE (UUID)
);

-- Default resource group
INSERT INTO RESOURCE_GROUPS(UUID, RESOURCE_GROUP_NAME, RESOURCE_GROUP_DSP_NAME) VALUES ('a52e934a-9fd9-44cb-9db1-716dcd13aae3', 'DFLTRSCGRP', 'DfltRscGrp');
-- Owner - Default resource group
INSERT INTO SEC_OBJECT_PROTECTION (OBJECT_PATH, CREATOR_IDENTITY_NAME, OWNER_ROLE_NAME, SECURITY_TYPE_NAME)
    VALUES ('/resourcegroups/DFLTRSCGRP', 'SYSTEM', 'SYSADM', 'SHARED');
-- ACL - Default storage pool definition
INSERT INTO SEC_ACL_MAP (OBJECT_PATH, ROLE_NAME, ACCESS_TYPE)
    VALUES ('/resourcegroups/DFLTRSCGRP', 'PUBLIC', 7);
INSERT INTO SEC_ACL_MAP (OBJECT_PATH, ROLE_NAME, ACCESS_TYPE)
    VALUES ('/resourcegroups/DFLTRSCGRP', 'USER', 7);
