package com.linbit.linstor.storage.data.adapter.nvme;

import com.linbit.linstor.api.pojo.NvmeRscPojo.NvmeVlmPojo;
import com.linbit.linstor.core.objects.AbsResource;
import com.linbit.linstor.core.objects.AbsVolume;
import com.linbit.linstor.security.AccessContext;
import com.linbit.linstor.storage.interfaces.categories.resource.VlmDfnLayerObject;
import com.linbit.linstor.storage.interfaces.layers.State;
import com.linbit.linstor.storage.interfaces.layers.nvme.NvmeVlmObject;
import com.linbit.linstor.storage.kinds.DeviceLayerKind;
import com.linbit.linstor.transaction.BaseTransactionObject;
import com.linbit.linstor.transaction.TransactionList;
import com.linbit.linstor.transaction.TransactionObjectFactory;
import com.linbit.linstor.transaction.manager.TransactionMgr;

import javax.annotation.Nullable;
import javax.inject.Provider;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class NvmeVlmData<RSC extends AbsResource<RSC>>
    extends BaseTransactionObject implements NvmeVlmObject<RSC>
{
    // unmodifiable data, once initialized
    private final AbsVolume<RSC> vlm;
    private final NvmeRscData<RSC> rscData;

    // not persisted, serialized, ctrl and stlt
    private long allocatedSize = UNINITIALIZED_SIZE;
    private String devicePath;
    private long usableSize = UNINITIALIZED_SIZE;

    // not persisted, not serialized, stlt only
    private boolean exists;
    private boolean failed;
    private boolean hasDisk;
    private final TransactionList<NvmeVlmData<RSC>, State> states;
    private Size sizeState;
    private String diskState;
    private long originalSize = UNINITIALIZED_SIZE;

    public NvmeVlmData(
        AbsVolume<RSC> vlmRef,
        NvmeRscData<RSC> rscDataRef,
        TransactionObjectFactory transObjFactoryRef,
        Provider<? extends TransactionMgr> transMgrProvider
    )
    {
        super(transMgrProvider);
        vlm = vlmRef;
        rscData = rscDataRef;
        exists = false;
        failed = false;
        states = transObjFactoryRef.createTransactionList(this, new ArrayList<>(), null);
        transObjs = Arrays.asList(
            vlm,
            rscData,
            states
        );
    }

    @Override
    public boolean exists()
    {
        return exists;
    }

    public void setExists(boolean existsRef)
    {
        exists = existsRef;
    }

    @Override
    public boolean hasFailed()
    {
        return failed;
    }

    public void setFailed(boolean failedRef)
    {
        failed = failedRef;
    }

    @Override
    public long getOriginalSize()
    {
        return originalSize;
    }

    @Override
    public void setOriginalSize(long originalSizeRef)
    {
        originalSize = originalSizeRef;
    }

    @Override
    public long getAllocatedSize()
    {
        return allocatedSize;
    }

    @Override
    public void setAllocatedSize(long allocatedSizeRef)
    {
        allocatedSize = allocatedSizeRef;
    }

    @Override
    public long getUsableSize()
    {
        return usableSize;
    }

    @Override
    public void setUsableSize(long usableSizeRef)
    {
        if (usableSizeRef != usableSize)
        {
            if (usableSize < usableSizeRef)
            {
                sizeState = Size.TOO_SMALL;
            }
            else
            {
                sizeState = Size.TOO_LARGE;
            }
        }
        else
        {
            sizeState = Size.AS_EXPECTED;
        }
        usableSize = usableSizeRef;
    }

    @Override
    public String getDevicePath()
    {
        return devicePath;
    }

    public void setDevicePath(String devicePathRef)
    {
        devicePath = devicePathRef;
    }

    @Override
    public Size getSizeState()
    {
        return sizeState;
    }

    public void setSizeState(Size sizeStateRef)
    {
        sizeState = sizeStateRef;
    }

    @Override
    public List<? extends State> getStates()
    {
        return states;
    }

    @Override
    public DeviceLayerKind getLayerKind()
    {
        return DeviceLayerKind.NVME;
    }

    @Override
    public String getDiskState()
    {
        return diskState;
    }

    public void setDiskState(String diskStateRef)
    {
        diskState = diskStateRef;
    }

    public boolean hasDisk()
    {
        return hasDisk;
    }

    @Override
    public AbsVolume<RSC> getVolume()
    {
        return vlm;
    }

    @Override
    public @Nullable VlmDfnLayerObject getVlmDfnLayerObject()
    {
        return null;
    }

    @Override
    public NvmeRscData<RSC> getRscLayerObject()
    {
        return rscData;
    }

    @Override
    public String getIdentifier()
    {
        return rscData.getSuffixedResourceName() + "/" + getVlmNr().getValue();
    }

    @Override
    public NvmeVlmPojo asPojo(AccessContext accCtxRef)
    {
        return new NvmeVlmPojo(
            getVlmNr().getValue(),
            devicePath,
            getBackingDevice(),
            allocatedSize,
            usableSize,
            diskState
        );
    }
}
